/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : February, 1997                                                 */
/* File Name : FuzzyVariable.cc                                          */
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <limits.h>
#include "FuzzyVariable.h"
#include "FuzzySet.h"
#include "Function.h"
#include "MembershipFunction.h"
#include <iostream.h>

FuzzyVariable::FuzzyVariable()
{
   index = 0;
   myClasses = NULL;
}

FuzzyVariable::FuzzyVariable(FuzzySet NewClass)
{  
   index = 1;
   myClasses = new FuzzySet[index];
   myClasses[0] = NewClass;
   
}// end of constructor


FuzzyVariable::~FuzzyVariable()
{  if(index) delete []  myClasses;
}// end of destructor

/*
    allows you to add a new class definition for this variable.
*/

void FuzzyVariable::addClass(FuzzySet newClass)
{  FuzzySet *Temp = new FuzzySet[index+1];
   int i;
   for(i=0;i<index;i++) Temp[i]= myClasses[i];
   Temp[i]=newClass;
   //if(index) delete [] myClasses;
   index++;
   myClasses = Temp;
}// end of addClass()

/*
   returns the membership value of this variable according to given
   class at the given point.
*/

float FuzzyVariable::getValueWithClassIndex(float x,int k)
{  float ret = FLT_MIN;
   if(k<index&&k>=0) ret = (myClasses[k]).getMFValue(x);
   return ret;
}// end of getValueWithClassIndex()

/*
//  Returns the Class ID of the corresponding variable.
*/

FuzzyVariable::getClassNumber(float x)
{  int maxIdx=0, k;
   double maxValue;

   if(index)
   {  maxValue = (myClasses[maxIdx]).getMFValue(x);

      for(k=1;k<index;k++)
        if(maxValue<(myClasses[k]).getMFValue(x))
        {  maxValue = (myClasses[k]).getMFValue(x);
           maxIdx=k;
        }
      maxIdx++;
   }

   return maxIdx;

}// end of getClassNumber()

FuzzyVariable::getValueWithClassIndex(int x,int k)
{  int ret = INT_MIN;
   if(k<index&&k>=0) ret = (myClasses[k]).getMFValue(x);
   return ret;
}// end of getValueWithClassIndex()

/*
   returns a function who belongs to the given class and is trimmed 
   according to given cut value.
*/

Function *FuzzyVariable::copyWithCutOffClassIndex(float cut,int c_indx)
{  Function *ret = NULL;
   
   if(c_indx>=0 && c_indx<index)
   {   ret = (myClasses[c_indx]).cutOffCopyMF(cut);
   }// end of if

   return ret;

}// end of copyWithCutOffClassIndex()

FuzzyVariable FuzzyVariable::operator=(const FuzzyVariable& RHS)
{  if(index) delete [] myClasses;
   index = RHS.index;
   myClasses = new FuzzySet[index];
   for(int i=0;i<index;i++) myClasses[i] = RHS.myClasses[i];
   return (*this);
}// end of operator =()

ostream& operator<<(ostream& os, const FuzzyVariable& RHS)
{  os << " FuzzyVariable has the following classes" << endl; 
   for(int i=0;i<RHS.index;i++) os << RHS.myClasses[i];
   os << endl;
   return os;
}// end of operator<<

